//
//  RotateAnimationView.m
//  RotateAnimation
//
//  Created by 张奥 on 2020/4/4.
//  Copyright © 2020 张奥. All rights reserved.
//

#import "RotateAnimationView.h"
#define UIColorFromRGB(rgbValue) [UIColor colorWithRed:((float)((rgbValue & 0xFF0000) >> 16))/255.0 \
green:((float)((rgbValue & 0xFF00) >> 8))/255.0 \
blue:((float)(rgbValue & 0xFF))/255.0 \
alpha:1.0]
@interface RotateAnimationView()

@property (nonatomic, strong)UIView *circleView;

@property (nonatomic,strong)CAShapeLayer *backLayer;
@property (nonatomic,strong)CAShapeLayer *progressLayer;

//单前的进度
@property (nonatomic,assign)CGFloat currentProgress;
//开始角度
@property (nonatomic,assign)CGFloat startAngle;

@property (nonatomic, strong)UIButton *hotImageView;

//是否是来自大头像
@property (nonatomic, assign)BOOL isBigAvart;

@end

@implementation RotateAnimationView

/*
// Only override drawRect: if you perform custom drawing.
// An empty implementation adversely affects performance during animation.
- (void)drawRect:(CGRect)rect {
    // Drawing code
}
*/

-(instancetype)initWithFrame:(CGRect)frame pathWidth:(CGFloat)pathWidth markWidth:(CGFloat)markWidth animationTime:(CGFloat)animationTime strokeColor:(UIColor *)strokeColor{
    self = [super initWithFrame:frame];
    if (self) {
        self.frame = frame;
        self.pathWidth = pathWidth;
        self.markWidth = markWidth;
        self.animationTime = animationTime;
        self.strokeColor = strokeColor;
        [self createUI];
    }
    return self;
}

-(void)createUI{
    self.currentProgress = 0;
    self.startAngle = 1.5*M_PI;
    float centerX = self.bounds.size.width/2.0;
    float centerY = self.bounds.size.height/2.0;
    //半径
    float radius = (self.bounds.size.width-self.pathWidth)/2.0;
    //贝塞尔路径
    UIBezierPath *path = [UIBezierPath bezierPathWithArcCenter:CGPointMake(centerX, centerY) radius:radius startAngle:(1.5f*M_PI) endAngle:-0.5f*M_PI clockwise:NO];
    //背景圆环
    CAShapeLayer *backLayer = [CAShapeLayer layer];
    self.backLayer = backLayer;
    backLayer.frame = self.bounds;
    backLayer.fillColor =  [[UIColor clearColor] CGColor];
    backLayer.strokeColor = self.strokeColor.CGColor;
    backLayer.lineWidth = self.pathWidth;
    backLayer.path = [path CGPath];
    backLayer.strokeEnd = 1.0;
    [self.layer addSublayer:backLayer];
    
    //进度layer
    CAShapeLayer *progressLayer = [CAShapeLayer layer];
    self.progressLayer = progressLayer;
    progressLayer.frame = self.bounds;
    progressLayer.fillColor =  [[UIColor clearColor] CGColor];
    progressLayer.strokeColor = [UIColor whiteColor].CGColor;
    progressLayer.lineWidth = self.pathWidth;
    progressLayer.path = [path CGPath];
    progressLayer.lineCap = kCALineCapRound;
    progressLayer.strokeEnd = 0.0;
    [self.layer addSublayer:progressLayer];
    
    //设置渐变颜色
    CAGradientLayer *gradientLayer =  [CAGradientLayer layer];
    gradientLayer.frame = self.bounds;
    [gradientLayer setColors:@[(id)[[UIColor redColor] CGColor],(id)[[UIColor blueColor] CGColor]]];
    gradientLayer.startPoint = CGPointMake(0, 0);
    gradientLayer.endPoint = CGPointMake(0, 1);
    [gradientLayer setMask:progressLayer]; //用progressLayer来截取渐变层
    [self.layer addSublayer:gradientLayer];
    

    //火焰图片
    UIButton *hotImageView = [[UIButton alloc] initWithFrame:CGRectMake((self.frame.size.width - self.markWidth)/2.f , -(self.markWidth - self.pathWidth)/2, self.markWidth, self.markWidth)];
    self.hotImageView = hotImageView;
    [hotImageView setImage:[UIImage imageNamed:@"xx"] forState:UIControlStateNormal];
    [hotImageView addTarget:self action:@selector(clickFireBtn) forControlEvents:UIControlEventTouchUpInside];
    [self addSubview:hotImageView];
    
}
-(void)setProgress:(CGFloat)progress clockwise:(BOOL)isclockwise{
    if (progress >= 1) {
        progress = 1;
    }
    if (progress <= 0) {
        progress = 0;
    }

    CGFloat spaceProgress = self.currentProgress - progress;
    [self doAnimationProgeress:progress spaceProgress:spaceProgress clockwise:isclockwise];
    self.currentProgress = progress;
}
-(void)doAnimationProgeress:(CGFloat)progress spaceProgress:(CGFloat)spaceProgress clockwise:(BOOL)isclockwise{
    if (progress == self.currentProgress)return;
    //绘制渐变色
    CABasicAnimation *animation = [CABasicAnimation animationWithKeyPath:@"strokeEnd"];
    animation.duration = 1.f;
    animation.timingFunction = [CAMediaTimingFunction functionWithName:kCAMediaTimingFunctionEaseInEaseOut];
    animation.fromValue = @(self.currentProgress);
    animation.toValue = @(progress);
    animation.repeatCount = 1;
    animation.removedOnCompletion = NO;
    animation.fillMode = kCAFillModeForwards;
    [self.progressLayer addAnimation:animation forKey:@"strokeEndAnimation"];

    //图片动画
    CGFloat value = spaceProgress * 2*M_PI;
    CGFloat endAngle = self.startAngle + value;
    UIBezierPath *bezierPath = [UIBezierPath bezierPathWithArcCenter:CGPointMake(self.frame.size.width  / 2, self.frame.size.height / 2) radius:(self.frame.size.width - self.pathWidth )/ 2 startAngle:self.startAngle endAngle:endAngle clockwise:isclockwise];
    CAKeyframeAnimation *hotAnimation = [CAKeyframeAnimation animationWithKeyPath:@"position"];
    hotAnimation.path = bezierPath.CGPath;
    hotAnimation.repeatCount = 1;
    hotAnimation.removedOnCompletion = NO;
    hotAnimation.fillMode = kCAFillModeForwards;
    hotAnimation.duration = 1;
    [self.hotImageView.layer addAnimation:hotAnimation forKey:nil];
    self.startAngle = endAngle;
    
}

-(void)clickFireBtn{
    if ([self.delegate respondsToSelector:@selector(clickFireBtn)]) {
        [self.delegate clickFireBtn];
    }
}
@end
